@@ -254,15 +254,18 @@ static int Dict_iterNext(JSOBJ obj, JSONTypeContext *tc)
       return 1;
     }
 
+    itemNameTmp = GET_TC(tc)->itemName;
     GET_TC(tc)->itemName = PyObject_Str(GET_TC(tc)->itemName);
+    Py_DECREF(itemNameTmp);
+    if (PyErr_Occurred())
+    {
+      PRINTMARK();
+      return -1;
+    }
     itemNameTmp = GET_TC(tc)->itemName;
     GET_TC(tc)->itemName = PyUnicode_AsUTF8String (GET_TC(tc)->itemName);
     Py_DECREF(itemNameTmp);
   }
-  else
-  {
-    Py_INCREF(GET_TC(tc)->itemName);
-  }
   PRINTMARK();
   return 1;
 }
@@ -334,6 +337,10 @@ static int SortedDict_iterNext(JSOBJ obj, JSONTypeContext *tc)
       else if (!PyBytes_Check(key))
       {
         key = PyObject_Str(key);
+        if (PyErr_Occurred())
+        {
+          goto error;
+        }
         keyTmp = key;
         key = PyUnicode_AsUTF8String(key);
         Py_DECREF(keyTmp);
@@ -648,6 +655,7 @@ DEFAULT:
     if (newObj)
     {
       PRINTMARK();
+      Py_XDECREF(pc->newObj);
       obj = pc->newObj = newObj;
       level += 1;
       goto BEGIN;
@@ -662,8 +670,15 @@ DEFAULT:
   PyErr_Clear();
 
   objRepr = PyObject_Repr(obj);
-  PyObject* str = PyUnicode_AsEncodedString(objRepr, "utf-8", "~E~");
-  PyErr_Format (PyExc_TypeError, "%s is not JSON serializable", PyBytes_AsString(str));
+  if (!objRepr)
+  {
+    goto INVALID;
+  }
+  PyObject* str = PyUnicode_AsEncodedString(objRepr, "utf-8", "strict");
+  if (str)
+  {
+    PyErr_Format (PyExc_TypeError, "%s is not JSON serializable", PyBytes_AsString(str));
+  }
   Py_XDECREF(str);
   Py_DECREF(objRepr);
 
