@@ -239,14 +239,15 @@ def test_encode_dict_values_ref_counting():
 @pytest.mark.skipif(
     hasattr(sys, "pypy_version_info"), reason="PyPy uses incompatible GC"
 )
-def test_encode_dict_key_ref_counting():
+@pytest.mark.parametrize("key", ["key", b"key", 1, True, None])
+@pytest.mark.parametrize("sort_keys", [False, True])
+def test_encode_dict_key_ref_counting(key, sort_keys):
     import gc
 
     gc.collect()
-    key = "key"
     data = {key: "abc"}
     ref_count = sys.getrefcount(key)
-    ujson.dumps(data)
+    ujson.dumps(data, sort_keys=sort_keys)
     assert ref_count == sys.getrefcount(key)
 
 
@@ -635,8 +636,14 @@ def test_dumps(test_input, expected):
 
 
 class SomeObject:
+    def __init__(self, message, exception=None):
+        self._message = message
+        self._exception = exception
+
     def __repr__(self):
-        return "Some Object"
+        if self._exception:
+            raise self._exception
+        return self._message
 
 
 @pytest.mark.parametrize(
@@ -644,13 +651,16 @@ class SomeObject:
     [
         (set(), TypeError, "set() is not JSON serializable"),
         ({1, 2, 3}, TypeError, "{1, 2, 3} is not JSON serializable"),
-        (SomeObject(), TypeError, "Some Object is not JSON serializable"),
+        (SomeObject("Some Object"), TypeError, "Some Object is not JSON serializable"),
+        (SomeObject("\ud800"), UnicodeEncodeError, None),
+        (SomeObject(None, KeyboardInterrupt), KeyboardInterrupt, None),
     ],
 )
 def test_dumps_raises(test_input, expected_exception, expected_message):
     with pytest.raises(expected_exception) as e:
         ujson.dumps(test_input)
-    assert str(e.value) == expected_message
+    if expected_message:
+        assert str(e.value) == expected_message
 
 
 @pytest.mark.parametrize(
@@ -974,6 +984,38 @@ def test_issue_334(indent):
     ujson.dumps(a, indent=indent)
 
 
+@pytest.mark.skipif(
+    hasattr(sys, "pypy_version_info"), reason="PyPy uses incompatible GC"
+)
+def test_default_ref_counting():
+    class DefaultRefCountingClass:
+        def __init__(self, value):
+            self._value = value
+
+        def convert(self):
+            if self._value > 1:
+                return type(self)(self._value - 1)
+            return 0
+
+    import gc
+
+    gc.collect()
+    ujson.dumps(DefaultRefCountingClass(3), default=lambda x: x.convert())
+    assert not any(
+        type(o).__name__ == "DefaultRefCountingClass" for o in gc.get_objects()
+    )
+
+
+@pytest.mark.parametrize("sort_keys", [False, True])
+def test_obj_str_exception(sort_keys):
+    class Obj:
+        def __str__(self):
+            raise NotImplementedError
+
+    with pytest.raises(NotImplementedError):
+        ujson.dumps({Obj(): 1}, sort_keys=sort_keys)
+
+
 """
 def test_decode_numeric_int_frc_overflow():
 input = "X.Y"
